#ifdef COMPRESS_H
#error Double include
#endif
#define COMPRESS_H

/*
 * $Source: /usr/cvsroot/cdx/src/cdx/CPacker.h,v $
 * $Author: MICHAELR $
 *
 * $Log: CPacker.h,v $
 * Revision 1.4  1999/08/29 22:29:04  MICHAELR
 * added CDXDLL support to classes/etc
 *
 * Revision 1.3  1999/07/17 10:22:28  Mindcry
 * Another small change
 *
 * Revision 1.2  1999/07/17 10:00:01  Mindcry
 * Changed the header. Hope it will show now the Revision and changeslist
 *
 * Revision 1.1  1999/07/17 11:56:00  Mindcry
 * Initial Revision to CVS.
 * for expand to Surface you need a special compressor!
 *
 * $Revision: 1.4 $
 */

#define MAXLIBSIZE 2500000
#define ALTLIB     0x40000000

#define N          4096 /* size of ring buffer */
#define F          64   /* upper limit for match_length */
#define THRESHOLD  2    /* encode string into position and length if match_length > threshold */
#define NIL        N    /* index for root of binary search trees */
#define M          15

#define Q1         (1UL << M)
#define Q2         (2 * Q1)
#define Q3         (3 * Q1)
#define Q4         (4 * Q1)
#define MAX_CUM    (Q1 - 1)
#define N_CHAR     (256 - THRESHOLD + F)

#define CalcChecksum(n) {                                         \
    for (i=0; i<3; i++) Buffer[textsize + i] = 0;                 \
    for (i=0; i<textsize; i += 4) (n) ^= * ((int*) (ramptr + i)); }

#define GetBit(buffer) {                       \
    if ((mask <<= 1) == 256) {                 \
        buffer = getc (fin); mask = 1;         \
    } value *= 2; if (buffer & mask) value ++; }

enum ResType {
    RES_BMP,
    RES_FILE,
    RES_FILENC,
    RES_FILET,
    RES_JPEG,
    RES_MIDI,
    RES_STR,
    RES_WAVE,
    RES_ANIM
};

class CPacker {
    public:     CPacker          (void);
    char*       ExpandToRam      (FILE*);
    void        InitBuffer       (void);
    unsigned    ReadBuffer       (const char*, const char* fileID = NULL);
    unsigned    ReadBuffer       (FILE*);
    unsigned    ReadBuffer       (void);
    unsigned    ReadLibFile      (const char*, unsigned);
    unsigned    ReadBufferPure   (const char*);
    unsigned    WriteBuffer      (const char*);
    unsigned    WriteBufferPure  (const char*);
    void        SeekTo           (unsigned);
    void        WriteB           (unsigned char);
    void        WriteW           (unsigned short);
    void        WriteD           (unsigned);
    unsigned    char  ReadB      (void);
    unsigned    short ReadW      (void);
    unsigned    ReadD            (void);
    void        Write            (void*, unsigned);
    void        Read             (void*, unsigned);
    void        WriteDmulti      (void*, unsigned);
    void        ReadDmulti       (void*, unsigned);
    void        ReadStr          (FILE*, char*);
    int         ReadNum          (FILE*);
    void        MakeMediaLibrary (const char*);
    unsigned    PackRamAppend    (BYTE*, DWORD, FILE*);
    void        InsertInLibrary  (FILE*, FILE*, const char*, UCHAR);
    DWORD       codesize,
                textsize;

    private:
    void        BufferCopy       (unsigned char*);
    void        Decode           (void);
    void        DeleteNode       (int);
    void        Encode           (void);
    void        EncodeChar       (int);
    void        EncodePosition   (int);
    void        InsertNode       (int);
    void        Output           (int);
    void        PutBit           (int);
    int         SpecRead         (void);
    void        UpdateModel      (int);
    unsigned    ReadBufCore      (void);

    ULONG       NumRefs,
                NumValid;
    BYTE        Buffer           [MAXLIBSIZE];
    void        *m_data;
    BYTE        *ramptr,
                text_buf         [N + F - 1];
    int         match_position,
                match_length,
                shifts,
                lson             [N + 1], 
                rson             [N + 257],
                dad              [N + 1],
                char_to_sym      [N_CHAR],
                sym_to_char      [N_CHAR + 1];
    DWORD       sym_freq         [N_CHAR + 1],
                sym_cum          [N_CHAR + 1],
                position_cum     [N + 1],
                stdtable         [(N_CHAR * 4) + N + 4],
                buffer,
                mask,
                low,
                high,
                value,
                filepos,
                pos;
    FILE        *fin,
                *fout;
};

extern CPacker pack;
